@@ -85,12 +85,15 @@ Billiard_connection_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 static void
 Billiard_connection_dealloc(BilliardConnectionObject* self)
 {
+    HANDLE handle = self->handle;
+
     if (self->weakreflist != NULL)
         PyObject_ClearWeakRefs((PyObject*)self);
 
-    if (self->handle != INVALID_HANDLE_VALUE) {
+    if (handle != INVALID_HANDLE_VALUE) {
+        self->handle = INVALID_HANDLE_VALUE;
         Py_BEGIN_ALLOW_THREADS
-        CLOSE(self->handle);
+        CLOSE(handle);
         Py_END_ALLOW_THREADS
     }
     PyObject_Del(self);
@@ -154,6 +157,7 @@ Billiard_connection_recvbytes(BilliardConnectionObject *self, PyObject *args)
     char *freeme = NULL;
     Py_ssize_t res, maxlength = PY_SSIZE_T_MAX;
     PyObject *result = NULL;
+    HANDLE handle;
 
     if (!PyArg_ParseTuple(args, "|" F_PY_SSIZE_T, &maxlength))
         return NULL;
@@ -171,10 +175,13 @@ Billiard_connection_recvbytes(BilliardConnectionObject *self, PyObject *args)
     if (res < 0) {
         if (res == MP_BAD_MESSAGE_LENGTH) {
             if ((self->flags & WRITABLE) == 0) {
-                Py_BEGIN_ALLOW_THREADS
-                CLOSE(self->handle);
-                Py_END_ALLOW_THREADS
-                self->handle = INVALID_HANDLE_VALUE;
+                handle = self->handle;
+                if (handle != INVALID_HANDLE_VALUE) {
+                    self->handle = INVALID_HANDLE_VALUE;
+                    Py_BEGIN_ALLOW_THREADS
+                    CLOSE(handle);
+                    Py_END_ALLOW_THREADS
+                }
             } else {
                 self->flags = WRITABLE;
             }
@@ -200,6 +207,7 @@ Billiard_connection_recvbytes_into(BilliardConnectionObject *self, PyObject *arg
     Py_ssize_t res, length, offset = 0;
     PyObject *result = NULL;
     Py_buffer pbuf;
+    HANDLE handle;
 
     CHECK_READABLE(self);
 
@@ -226,10 +234,13 @@ Billiard_connection_recvbytes_into(BilliardConnectionObject *self, PyObject *arg
     if (res < 0) {
         if (res == MP_BAD_MESSAGE_LENGTH) {
             if ((self->flags & WRITABLE) == 0) {
-                Py_BEGIN_ALLOW_THREADS
-                CLOSE(self->handle);
-                Py_END_ALLOW_THREADS
-                self->handle = INVALID_HANDLE_VALUE;
+                handle = self->handle;
+                if (handle != INVALID_HANDLE_VALUE) {
+                    self->handle = INVALID_HANDLE_VALUE;
+                    Py_BEGIN_ALLOW_THREADS
+                    CLOSE(handle);
+                    Py_END_ALLOW_THREADS
+                }
             } else {
                 self->flags = WRITABLE;
             }
@@ -267,6 +278,7 @@ Billiard_connection_recvbytes_into(BilliardConnectionObject *self, PyObject *arg
     char *freeme = NULL, *buffer = NULL;
     Py_ssize_t length = 0, res, offset = 0;
     PyObject *result = NULL;
+    HANDLE handle;
 
     CHECK_READABLE(self);
 
@@ -289,10 +301,13 @@ Billiard_connection_recvbytes_into(BilliardConnectionObject *self, PyObject *arg
     if (res < 0) {
        if (res == MP_BAD_MESSAGE_LENGTH) {
             if ((self->flags & WRITABLE) == 0) {
-                Py_BEGIN_ALLOW_THREADS
-                CLOSE(self->handle);
-                Py_END_ALLOW_THREADS
-                self->handle = INVALID_HANDLE_VALUE;
+                handle = self->handle;
+                if (handle != INVALID_HANDLE_VALUE) {
+                    self->handle = INVALID_HANDLE_VALUE;
+                    Py_BEGIN_ALLOW_THREADS
+                    CLOSE(handle);
+                    Py_END_ALLOW_THREADS
+                }
             } else {
                 self->flags = WRITABLE;
             }
@@ -418,6 +433,7 @@ Billiard_connection_recv_payload(BilliardConnectionObject *self)
     char *freeme = NULL;
     Py_ssize_t res;
     PyObject *view = NULL;
+    HANDLE handle;
 
     CHECK_READABLE(self);
 
@@ -426,10 +442,13 @@ Billiard_connection_recv_payload(BilliardConnectionObject *self)
     if (res < 0) {
         if (res == MP_BAD_MESSAGE_LENGTH) {
             if ((self->flags & WRITABLE) == 0) {
-                Py_BEGIN_ALLOW_THREADS
-                CLOSE(self->handle);
-                Py_END_ALLOW_THREADS
-                self->handle = INVALID_HANDLE_VALUE;
+                handle = self->handle;
+                if (handle != INVALID_HANDLE_VALUE) {
+                    self->handle = INVALID_HANDLE_VALUE;
+                    Py_BEGIN_ALLOW_THREADS
+                    CLOSE(handle);
+                    Py_END_ALLOW_THREADS
+                }
             } else {
                 self->flags = WRITABLE;
             }
@@ -456,6 +475,7 @@ Billiard_connection_recv_obj(BilliardConnectionObject *self)
     char *freeme = NULL;
     Py_ssize_t res;
     PyObject *temp = NULL, *result = NULL;
+    HANDLE handle;
 
     CHECK_READABLE(self);
 
@@ -465,10 +485,13 @@ Billiard_connection_recv_obj(BilliardConnectionObject *self)
     if (res < 0) {
         if (res == MP_BAD_MESSAGE_LENGTH) {
             if ((self->flags & WRITABLE) == 0) {
-                Py_BEGIN_ALLOW_THREADS
-                CLOSE(self->handle);
-                Py_END_ALLOW_THREADS
-                self->handle = INVALID_HANDLE_VALUE;
+                handle = self->handle;
+                if (handle != INVALID_HANDLE_VALUE) {
+                    self->handle = INVALID_HANDLE_VALUE;
+                    Py_BEGIN_ALLOW_THREADS
+                    CLOSE(handle);
+                    Py_END_ALLOW_THREADS
+                }
             } else {
                 self->flags = WRITABLE;
             }
@@ -545,11 +568,13 @@ Billiard_connection_fileno(BilliardConnectionObject* self)
 static PyObject *
 Billiard_connection_close(BilliardConnectionObject *self)
 {
-    if (self->handle != INVALID_HANDLE_VALUE) {
-        Py_BEGIN_ALLOW_THREADS
-        CLOSE(self->handle);
-        Py_END_ALLOW_THREADS
+    HANDLE handle = self->handle;
+
+    if (handle != INVALID_HANDLE_VALUE) {
         self->handle = INVALID_HANDLE_VALUE;
+        Py_BEGIN_ALLOW_THREADS
+        CLOSE(handle);
+        Py_END_ALLOW_THREADS
     }
 
     Py_RETURN_NONE;
